import copy
import random
class Matrix:
    '''基于列表实现的矩阵类'''
    def __init__(self, numRows, numCols, x=0):
        self.shape = (numRows, numCols)  #记录矩阵的行数和列数
        self.row = self.shape[0]
        self.column = self.shape[1]
        #生成self.row行self.column列的矩阵元素全为x
        self.matrix = [[x for col in range(self.column)] for row in range(self.row)] 
        
    # 返回矩阵A的元素A(i,j)的值: matrix [i-1][ j-1]
    def __getitem__(self, index):
        if isinstance(index, int):
            return self.matrix[index-1]
        elif isinstance(index, tuple) and len(index)==2:
            return self.matrix[index[0]-1][index[1]-1]
    # 设置矩阵matrix (i,j)的值为value，即matrix [i-1][ j-1] = value
    def __setitem__(self, index, value):
        if isinstance(index, int):
            self.matrix[index-1] = copy.deepcopy(value) #深度拷贝
        elif isinstance(index, tuple) and len(index)==2:
            self.matrix[index[0]-1][index[1]-1] = value
    def __eq__(self, B):        #判断两个矩阵是否相等
        '''B是一个Matrix类的对象'''
        if self.row==B.row and self.column== B.column:
            for i in range(self.row):
                for j in range(self.column):
                    if self.matrix[i][j]==B.matrix[i][j]:
                        pass
                    else:
                        return "两个矩阵不相等"
                return "两个矩阵相等"
        else:
            return "维度不同，两个矩阵不相等"

    def __add__(self, B):      #将两个矩阵相加
        '''矩阵加法，B是一个Matrix类的对象'''
        if self.shape == B.shape: #维度相同才能相加
            M = Matrix(self.row, self.column)  #临时生成一个和B维度相同的矩阵
            for i in range(self.row):
                for j in range(self.column):
                    M.matrix[i][j] = self.matrix[i][j] + B.matrix[i][j]
            return M.matrix
        else:
            return "维度不同，两个矩阵不能相加"
        
    def __sub__(self, B):      #将两个矩阵相减
        '''矩阵减法，参数B是一个Matrix类的对象'''
        if self.shape == B.shape: #维度相同才能相减
            M = Matrix(self.row, self.column)  #临时生成一个和B维度相同的矩阵
            for i in range(self.row):
                for j in range(self.column):
                    M.matrix[i][j] = self.matrix[i][j] - B.matrix[i][j]
            return M.matrix
        else:
            return "维度不同，两个矩阵不能相减"
    
    def __mul__(self, B):      #将两个矩阵相乘
        '''矩阵乘法，参数B是一个数或是一个Matrix类的对象'''
        if isinstance(B, int) or isinstance(B,float):  #B是一个数
            M = Matrix(self.row, self.column)
            for i in range(self.row):
                for j in range(self.column):
                    M.matrix[i][j] = self.matrix[i][j]*B
            return M.matrix
        #一个矩阵的列数等于另一个矩阵的行数，两矩阵才能相乘
        elif self.column == B.row: 
            M = Matrix(self.row, B.column)
            for i in range(self.row):
                for j in range(B.column):
                    sum = 0
                    for k in range(self.column):
                        sum += self.matrix[i][k] * B.matrix[k][j]
                    M.matrix[i][j] = sum
            return M.matrix
        else:
            return "两个矩阵不能相乘"

    def  __sum_of_elements__(self):  #对矩阵的所有元素求和
        total = 0
        for i in self.matrix:
            for j in i:
                total += j
        return total
        
    def maxRow(self):  #找出和最大的行
        max_row = sum(self.matrix[0])
        index_of_max_row = 0
        for row in range(1,self.row):
            if sum(self.matrix[row]) > max_row:
                max_row = sum(self.matrix[row])
                index_of_max_row = row
        print("和最大行的索引：",index_of_max_row)
        print("和最大行的和：",max_row)
    def __random__(self):    # 打乱矩阵的所有元素
        for row in range(self.row):
            for column in range(self.column):
                i = random.randint(0, self.row - 1)
                j = random.randint(0, self.column - 1)
                self.matrix[row][column], self.matrix[i][j] = self.matrix[i][j], self.matrix[row][column]
        return self.matrix
        
    def transpose(self):      #对矩阵进行转置
        '''矩阵转置'''
        M = Matrix(self.column, self.row)
        for i in range(self.column):
            for j in range(self.row):
                M.matrix[j][i] = self.matrix[i][j]
            return M.matrix

    def show(self):          #打印矩阵
        '''打印矩阵'''
        for i in range(self.row):
            for j in range(self.column):
                print(self.matrix[i][j],end=' ')
            print()




